//
//  CHDatabase.h
//  StickyBrainKit
//
//  Created by Rod Schmidt on 3/24/2005.
//  Copyright 2005 Chronos, LC. All rights reserved.
//

#import <Foundation/Foundation.h>

@class CHDatabase, OpenBase, DBRecord, OpenBaseAdminConnection, SBSpotlight;

extern NSString * const CH_DB_VERSION;

enum
{
	CHDatabaseHostPasswordFailed = 10000,
	CHDatabaseReadPasswordFailed,
	CHDatabaseConversionFailed,
	CHDatabaseAdminConnectionFailed
};

// Sent when the database is connected to.
#define CHDatabaseDidConnectNotification	@"CHDatabaseDidConnectNotification"
// When we disconnect, we send this:
#define CHDatabaseWillDisconnectNotification	@"CHDatabaseWillDisconnectNotification"
// When we convert:
extern NSString * const CHDatabaseWillConvertNotification;

void CHDatabaseError(CHDatabase* db, NSString* titleKey, NSString* msgKey);

// Keys for dictionaries inside _sortData:
extern NSString * const keyListViewSortColumnName;  // NSString corresponding to a PGFieldString
extern NSString * const keyListViewSortUp;			// NSNumber containing a BOOL

@interface CHDatabase : NSObject 
{
	OpenBase*   _db;
	NSRecursiveLock *_dbLock;
	NSString	*_persistentDatabaseID;
	NSString	*_databaseID;
	NSString*   _host;
	NSString*   _dbName;
	NSString*   _userID;
	NSString*   _password;
	NSString*	_hostPassword;
	NSString*	_readPassword;
	NSString*	_softwareId;
	BOOL		_isAdmin;
	BOOL		_checkedAdmin;
	int			_errorCode;
	NSString*   _errorMessage;
	BOOL		_initializing;
	BOOL		_readSmallIcons;
	BOOL		_sortDescending;
	BOOL		_isLocalReplica;
	id			_delegate;
	NSMutableSet *_notificationHandlers;
	NSRecursiveLock *_notificationHandlersLock;
	
	NSMutableArray*	_sortData;
	id				_cachedMe;
	NSMutableArray	*_cachedColors;
	NSMutableArray*	_cachedLabels;
	long long		_whiteColorID;
	NSMutableDictionary	*_cachedUserIDsForRowIDs;
	NSMutableDictionary *_cachedRowIDsForUserIDs;
	NSMutableDictionary *_folderNameToIDMap;
	BOOL			_needToLoginAsAdmin;	// YES if we are converting an old database that doesn't have a chronosadmin user
	NSMutableDictionary*	_cachedFolderPrivileges;
	NSMutableDictionary *_cachedCalendarPrivileges;
	BOOL	_createDatabaseIfNonExistent;
	BOOL	_creatingFirstDefaultFolder;
}

// Class methods
//////////////////

+(BOOL) isNewDatabase: (OpenBase*) openDb;
+(BOOL) isOBInstalled;
+(BOOL) isOBRunning;
+(NSString*) localDatabaseName;

// Initialization methods
//////////////////////////

- (void)setCreateDatabaseIfNonExistent:(BOOL)aFlag;

-(id) initWithDatabaseName: (NSString*) aName 
					  host: (NSString*) host 
					userID: (NSString*) userID 
				  password: (NSString*) password;

// connectionInfo must contain the following keys: "databaseName", "hostName", "userName", "password"
-(id) initWithConnectionInfo: (NSDictionary*) connectionInfo;

-(id) initWithConnectionInfoFromDatabase: (CHDatabase*) anotherDb;
-(id) initLocalDatabase;
- (id)initLocalReplicaOfRemoteDatabaseWithName:(NSString *)name;

// Creation methods
////////////////////

-(BOOL) createDatabase;
-(BOOL) configure: (OpenBaseAdminConnection*) adminConnection;
-(BOOL) isEmpty;

// Connection methods
//////////////////////

-(BOOL) exists;
-(BOOL) connectDB: (NSString*) password;
-(BOOL) connect;
-(void) disconnect;
-(BOOL) isConnected;
-(OpenBaseAdminConnection*) adminConnectionWithValidatedHostPassword:(BOOL)validateHostPassword readPassword:(BOOL)validateReadPassword;
-(BOOL) isValid;	// Is this a valid Chronos database?

// Tell the database to connect as admin. This is need to connect to old databases
-(void) setNeedToLoginAsAdmin;

// Accessors
//////////////

-(void) setSoftwareId: (NSString*) softwareID;

-(NSString*) name;
-(NSString*) hostName;
- (NSString *)hostNameForMasterDatabase;
-(NSString*) userID;

/*	We have two different database identifiers. Here is the difference between the two:
	-serverIDString returns the database identifier that is used by the OpenBase database system.
	-persistentDatabaseID returns the database identifier stored inside the Settings table
		located inside our database.
	
	As a general rule, -serverIDString should be used to uniquely identify databases, since
	OpenBase internally uses this value to identify databases (during notifications, etc.).
	-persistentDatabaseID should **only** be used to identify databases when the identifier
	must survive a reboot, because the IDs returned by -serverIDString are reset every time
	the OpenBase "openexec" process is launched (usually during a reboot). */
- (NSString *)serverIDString;
- (NSString *)persistentDatabaseID;

-(BOOL) setPassword;
- (void)setDatabasePassword:(NSString*)password;

-(NSString*) hostPassword;
-(void) setHostPassword: (NSString*) password;

-(NSString*) readPassword;
-(void) setReadPassword: (NSString*) password;

- (BOOL)isLocalReplica;

-(BOOL) usingSmallIcons;
-(void) setUseSmallIcons: (BOOL) useSmallIcons;

-(id) delegate;
-(void) setDelegate: (id) aDelegate;

// User methods
////////////////

-(BOOL) isAdmin;
-(BOOL) isAdmin: (NSString*) userID;
- (BOOL)verifyPassword:(NSString*)testPassword;

// SQL execution methods
/////////////////////////

//-(NSString*) escapeSQLString: (NSString*) aString;
-(BOOL) executeCommand;
-(BOOL) saveRecord: (DBRecord*) record;
-(BOOL) deleteRecord: (DBRecord*) record;
-(BOOL) markRow: (long long) rowID inTable: (NSString*) tableName;
-(BOOL) unmarkRow: (long long) rowID inTable: (NSString*) tableName;
- (NSData *)retrieveBinaryWithBlobID:(NSString *)blobID;

// Error handling methods
//////////////////////////

-(int) errorCode;
-(NSString*) errorMessage;
-(void) setErrorMessage: (NSString*) msg;

// Transaction methods
///////////////////////

-(void) beginTransaction;
-(BOOL) commit;
-(void) rollback;

// Notification methods
////////////////////////

-(void) startNotificationFor: (id) observer;
-(void) removeNotificationFor: (id) observer;

// Keychain methods
////////////////////

-(BOOL) addToKeychain;
- (BOOL)updateKeychain;

+(NSString*) passwordForDatabase: (NSString*) dbName host: (NSString*) host login: (NSString*) login;
+(NSString*) hostPasswordForDatabase: (NSString*) dbName host: (NSString*) host login: (NSString*) login;
+(NSString*) readPasswordForDatabase: (NSString*) dbName host: (NSString*) host login: (NSString*) login;
+(BOOL) removeFromKeychain: (NSString*) dbName host: (NSString*) host login: (NSString*) login;

// Replication
//////////////
- (void)setupReplicationToDatabaseOnHostName:(NSString *)hostName databaseName:(NSString *)dbName;
- (void)replicate;
- (void)teardownReplicationToDatabaseOnHostName:(NSString *)hostName databaseName:(NSString *)dbName;

// Misc.
////////
- (NSComparisonResult)compare:(CHDatabase *)db;

@end

@interface NSObject (CHDatabaseDelegate)
-(NSString*) database: (CHDatabase*) db hostPasswordForAdminConnection: (OpenBaseAdminConnection*) adminConnection;
-(NSString*) database: (CHDatabase*) db readPasswordForAdminConnection: (OpenBaseAdminConnection*) adminConnection;
@end
