//
//  SBLibrary.h
//  CHDatabaseKit
//
//  Created by Rod Schmidt on 8/24/06.
//  Copyright 2006 Chronos, LC. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "CHTypes.h"
#import "CHDatabase+Notes.h"

@class CHDatabase, PGContactsModel, PGPerson;
@class SBIconCache;
@class SBFolderCache;
@class SBNotesCache;
@class SBFormsCache;
@class SBKeywordsCache;

// Represents a library in SOHO Notes. A Library is a collection of folders and notes and anything
// that goes with them, including users and their privileges to access the folders and notes.
//
@interface SBLibrary : NSObject 
{
	CHDatabase *            _database;
    SBFolder *              _root;	// The root of the library's folder hierarchy
	NSMutableDictionary *   _folderAccessLevels;
	SBIconCache *			_folderIconsCache;
	SBFolderCache *			_folderCache;
	SBNotesCache *			_notesCache;
	SBFormsCache *			_formsCache;
	SBKeywordsCache *		_keywordsCache;
	NSMetadataQuery *		_query;
	NSMutableArray *		_queryResults;
	NSUInteger				_queryResultsNdx;
	NSString *				_privateNotesPasswordFromUser;
	
	// SmartFolder query holders;
	id		_queryContact;
	BOOL	_queryPrivateNotes;
	BOOL	_queryFlaggedNotes;
}

// Initialization
///////////////////

+(SBLibrary *)libraryWithDatabase: (CHDatabase *)database;

-(id) initWithDatabase: (CHDatabase *) database;

// General methods
///////////////////

/*  We have two different database identifiers. Here is the difference between the two:
      -databaseID returns the database identifier that is used by the OpenBase database system.
      -persistentDatabaseID returns the database identifier stored inside the Settings table located inside our database.

    As a general rule, -databaseID should be used to uniquely identify databases, since	OpenBase internally uses this 
    value to identify databases (during notifications, etc.).
    -persistentDatabaseID should **only** be used to identify databases when the identifier	must survive a reboot, 
    because the IDs returned by -databaseID are reset every time the OpenBase "openexec" process is launched 
    (usually during a reboot). 
*/
-(NSString *)databaseID;
-(NSString *)persistentDatabaseID;

-(NSString *)hostName;
-(NSString *)name;

-(id)delegate;
-(void)setDelegate: (id)aDelegate;

-(BOOL) validFolder: (SBFolder *)folder;

// Cache
/////////

-(BOOL)notesAreCached;
-(void)resetAllCaches;

// Contacts
///////////

-(PGContactsModel *)contactsModel;

// Database methods
////////////////////

-(BOOL)connect;
-(CHDatabase *)database;
-(BOOL)defaultFoldersExist;
-(void)disconnect;
-(int)errorCode;
-(NSString *)errorMessage;
-(BOOL)exists;
-(long long)folderTypeID: (NSString *)folderTypeName;
-(BOOL)hasDatabase: (CHDatabase *)db;
-(BOOL)hasDatabaseWithID: (NSString *)databaseID userID: (NSString *)userID;
-(BOOL)isConnected;
-(BOOL)isLocal;
-(BOOL)isLocalReplica;
-(BOOL)isValid;
-(BOOL)manualSortNeededForColumn: (NSString *)columnName;
-(BOOL)matchesConnectionInfo: (NSDictionary *)connectionInfo;
-(void)setCreateDatabaseIfNonExistent: (BOOL)aFlag;
// Tell the library to connect as admin. This is need to connect to old libraries
-(void) setNeedToLoginAsAdmin;
-(NSString *)userID;

// Hot Keys
////////////

-(BOOL) isHotKeyInUse: (short) keyCode modifiers: (long) modifiers;

// Settings
///////////

-(NSDictionary *)loadSettings;
-(BOOL)updateSettings: (NSDictionary *)settings;

-(BOOL)matchesPrivateNotesPassword: (NSString *)aPassword;
-(BOOL)hasPrivateNotesPassword;
-(NSString *)privateNotesPassword;
-(void)savePrivateNotesPassword:(NSString *)newPassword;

-(NSString *)privateNotesPasswordFromUser;
-(void)setPrivateNotesPasswordFromUser:(NSString *)password;

@end
